@@ -3,21 +3,19 @@
 
 #define _USE_MATH_DEFINES
 
-
-#include <boost/math/special_functions/digamma.hpp>
-#include <rbobyqa.h>
 #include "general.h"
 #include "maths.h"
 #include "openmpheader.h"
 #include "covariance.hpp"
 #include "linearpredictor.hpp"
+#include "calculator.hpp"
 #include "sparse.h"
-#include <random>
 
 // [[Rcpp::depends(BH)]]
 // [[Rcpp::depends(RcppEigen)]]
 // [[Rcpp::plugins(openmp)]]
 
+
 namespace glmmr {
 
 using namespace rminqa;
@@ -28,10 +26,12 @@ public:
   glmmr::Formula formula_;
   glmmr::Covariance covariance_;
   glmmr::LinearPredictor linpred_;
+  ArrayXd variance_;
   double var_par_;
   std::string family_; 
   std::string link_;
   VectorXd offset_;
+  ArrayXd weights_;
   const VectorXd y_;
   int n_;
   int Q_;
@@ -49,10 +49,12 @@ public:
   ) : formula_(formula),
       covariance_(formula_,data,colnames),
       linpred_(formula_,data,colnames),
+      variance_(ArrayXd::Constant(data.rows(),1.0)),
       var_par_(1.0),
       family_(family),
       link_(link),
       offset_(VectorXd::Zero(data.rows())),
+      weights_(ArrayXd::Constant(data.rows(),1.0)),
       y_(y),
       n_(data.rows()),
       Q_(covariance_.Q()),
@@ -81,112 +83,68 @@ public:
           upper_t_.push_back(R_PosInf);
         }
         gen_sigma_blocks();
+        setup_calculator();
       };
   
   void set_offset(const VectorXd& offset);
-  
+  void set_weights(const ArrayXd& weights);
   void update_beta(const VectorXd &beta);
-  
   void update_beta(const dblvec &beta);
-  
   void update_beta_extern(const dblvec &beta);
-  
   void update_theta(const VectorXd &theta);
-  
   void update_theta(const dblvec &theta);
-  
   void update_theta_extern(const dblvec &theta);
-
   void update_u(const MatrixXd &u);
-  
   void update_W();
-  
+  void update_var_par(const double& v);
+  void update_var_par(const ArrayXd& v);
   double log_prob(const VectorXd &v);
-  
-  VectorXd log_gradient(const VectorXd &v,
-                              bool beta = false);
- 
-  MatrixXd linpred(){
-    return (zu_.colwise()+(linpred_.xb()+offset_));
-  }
-  
-  
-  VectorXd xb(){
-    return linpred_.xb()+offset_;
-  }
-  
+  VectorXd log_gradient(const VectorXd &v,bool beta = false);
+  MatrixXd linpred();
+  VectorXd xb();
   double log_likelihood();
-  
   double full_log_likelihood();
-  
   void ml_theta();
-  
   void ml_beta();
-  
   void ml_all();
-  
   void laplace_ml_beta_u();
-  
   void laplace_ml_theta();
-  
   void laplace_ml_beta_theta();
-  
   void nr_beta();
-  
   void laplace_nr_beta_u();
-  
-  MatrixXd laplace_hessian(double tol = 1e-4);
-  
-  MatrixXd hessian(double tol = 1e-4);
-  
-  MatrixXd u(bool scaled = true){
-    if(scaled){
-      return covariance_.Lu(u_);
-    } else {
-      return u_;
-    }
-  }
-  
+  vector_matrix b_score();
+  vector_matrix re_score();
+  matrix_matrix hess_and_grad();
+  MatrixXd observed_information_matrix();
   MatrixXd Zu();
-  
   MatrixXd Sigma(bool inverse = false);
-  
   MatrixXd information_matrix();
-  
-  vector_matrix predict_re(const ArrayXXd& newdata_,
-               const ArrayXd& newoffset_);
-  
-  VectorXd predict_xb(const ArrayXXd& newdata_,
-                      const ArrayXd& newoffset_);
-  
-  void mcmc_sample(int warmup,
-                   int samples,
-                   int adapt = 100);
-  
+  vector_matrix predict_re(const ArrayXXd& newdata_,const ArrayXd& newoffset_);
+  VectorXd predict_xb(const ArrayXXd& newdata_,const ArrayXd& newoffset_);
+  MatrixXd sandwich_matrix();
+  void mcmc_sample(int warmup,int samples,int adapt = 100);
   void mcmc_set_lambda(double lambda);
-  
   void mcmc_set_max_steps(int max_steps);
-  
   void mcmc_set_refresh(int refresh);
-  
   void mcmc_set_target_accept(double target);
-  
   void set_trace(int trace);
-  
   double aic();
-  
   void make_covariance_sparse();
-  
   void make_covariance_dense();
-  
-  ArrayXd optimum_weights(double N, double sigma_sq, VectorXd C, double tol = 1e-5,
-                          int max_iter = 501);
+  std::vector<MatrixXd> sigma_derivatives();
+  MatrixXd information_matrix_theta();
+  matrix_matrix kenward_roger();
+  ArrayXd optimum_weights(double N, double sigma_sq, VectorXd C, double tol = 1e-5, int max_iter = 501);
+  MatrixXd u(bool scaled = true);
+  VectorXd W();
   
 private:
   ArrayXd size_m_array;
   ArrayXd size_q_array;
   ArrayXd size_n_array;
   ArrayXd size_p_array;
+  glmmr::calculator calc_;
+  glmmr::calculator vcalc_;
   sparse ZL_;
   MatrixXd u_;
   MatrixXd zu_;
@@ -211,27 +169,19 @@ private:
   double target_accept_ = 0.9;
   bool verbose_ = true;
   std::vector<glmmr::SigmaBlock> sigma_blocks_;
+  bool weighted_ = false;
   
+  void setup_calculator();
   void gen_sigma_blocks();
-  
   MatrixXd sigma_block(int b, bool inverse = false);
-  
   MatrixXd sigma_builder(int b, bool inverse = false);
-  
   MatrixXd information_matrix_by_block(int b);
-  
   dblvec get_start_values(bool beta, bool theta, bool var = true);
-  
   dblvec get_lower_values(bool beta, bool theta, bool var = true);
-  
   dblvec get_upper_values(bool beta, bool theta, bool var = true);
-  
-  VectorXd new_proposal(const VectorXd& u0_, bool adapt, 
-                        int iter, double rand);
-  
-  void sample(int warmup,
-                  int nsamp,
-                  int adapt = 100);
+  VectorXd new_proposal(const VectorXd& u0_, bool adapt,  int iter, double rand);
+  void calculate_var_par();
+  void sample(int warmup,int nsamp,int adapt = 100);
   
   class D_likelihood : public Functor<dblvec> {
     Model& M_;
@@ -273,7 +223,6 @@ private:
     double operator()(const dblvec &par);
   };
   
-  
   class LA_likelihood : public Functor<dblvec> {
     Model& M_;
     MatrixXd v;
@@ -329,12 +278,11 @@ private:
 
 }
 
-
-
-
-
 #include "likelihood.ipp"
-#include "mhmcmc.ipp"
-#include "model.ipp"
+#include "modelbasefunctions.ipp"
+#include "modelupdatefunctions.ipp"
+#include "modelfittingfunctions.ipp"
+#include "modelmcmcfunctions.ipp"
+#include "modeloptimweights.ipp"
 
 #endif
